/*
 * \brief  SPI rpc client
 * \author Jean-Adrien Domage <jean-adrien.domage@gapfruit.com>
 * \date   2021-04-28
 */

/*
 * Copyright (C) 2013-2021 Genode Labs GmbH
 * Copyright (C) 2021 gapfruit AG
 *
 * This file is part of the Genode OS framework, which is distributed
 * under the terms of the GNU Affero General Public License version 3.
 */

#ifndef _INCLUDE__SPI_CLIENT__CLIENT_H_
#define _INCLUDE__SPI_CLIENT__CLIENT_H_

#include <base/mutex.h>
#include <base/rpc_client.h>
#include <base/attached_dataspace.h>
#include <util/string.h>

#include <spi_session/spi_session.h>

namespace Spi {

	using namespace Genode;

	class Session_client;
}

class Spi::Session_client : public Rpc_client<Session>
{
private:

	Attached_dataspace _tx_buffer;
	Attached_dataspace _rx_buffer;


public:

	Session_client(Region_map &rm, Capability<Session> cap)
	:
		Rpc_client<Session> { cap },
		_tx_buffer          { rm, call<Rpc_tx_dataspace>() },
		_rx_buffer          { rm, call<Rpc_rx_dataspace>() }
	{ }

	void duplex_transfer(Transfer const &trxn) override
	{
		/* throw an exception if there are no buffer at all to operate on */
		/* throw an exception if we don't have enough memory to operate */
		if (trxn.buffer_size == 0
		    || (!trxn.tx_buffer || !trxn.rx_buffer)
		    || trxn.buffer_size > _tx_buffer.size()
		    || trxn.buffer_size > _rx_buffer.size()) {
			throw Spi::Io_buffer_too_small();
		}

		memcpy(_tx_buffer.local_addr<uint8_t>(), trxn.tx_buffer, trxn.buffer_size);
		call<Rpc_duplex_transfer>(trxn.buffer_size, trxn.max_burst_size, trxn.byte_reordering);
		memcpy(trxn.rx_buffer, _rx_buffer.local_addr<uint8_t>(), trxn.buffer_size);
	}


	void settings(Settings setting) override {
		call<Rpc_set_setting>(setting);
	}


	Settings settings() override {
		return call<Rpc_get_setting>();
	}
};

#endif // _INCLUDE__SPI_CLIENT__CLIENT_H_
